#define versionx "09_lcd_multiples"
/*                1234567890123456

Gestion de multiples (jusqu'à 8 ou 16) lcd de type (2 lignes de 16 caractères)

*/
 
//**********************************************************
// BIBLIOTHEQUES
// incluses dans le package Arduino de base
#include <LiquidCrystal.h>  // LCD

// ajoutées
#include <Streaming_ger.h>
#include <Time_ger.h>

//**********************************************************
// DECLARATIONS
// *** GENERAL *** 
/* DEBUG : 
	0=pas de debug
 1=debug minimal pour voir les étapes
 2=debug plus fin
*/
int debug=0;  
//----------------------------------------------
// LCD initialize the library with the numbers of the interface pins
/* Câblage du LCD "seul"
 * LCD pin 1 Vss to ARDUINO GND
 * LCD pin 2 Vdd to ARDUINO VCC 5V
 * LCD pin 3 Vo contraste sur le point milieu d'un potentiomètre de 10K entre le GND et le VCC (réglé très près de la masse)
 * LCD pin 4 RS Register Select to ARDUINO digital pin 4
 * LCD pin 5 RW Read/write to ARDUINO GND
 * LCD pin 6 Enable pin to ARDUINO digital pin 2 + MULIPLEXEUR type CMOS 4051
 * LCD pin 11 DB4 pin to ARDUINO digital pin 6
 * LCD pin 12 DB5 pin to ARDUINO digital pin 7
 * LCD pin 13 DB6 pin to ARDUINO digital pin 8
 * LCD pin 14 DB7 pin to ARDUINO digital pin 9
 

 * Câblage du shield avec MC14051 et les 8 LCD (connecteur HE10-16points)
 * ___ - __________- _________ - ____________
 * UNO - 4051      - HE10-16   - LCD
 * ___ - __________- _________ - ____________
 * 5V  - pin 16    - broche 1  - 1 (GND)
 * GND - pin 7,8   - broche 3  - 2 (5V)
 *                             - 3 (contraste) 1k vers GND
 * D4  -           - broche 5  - 4 (RS)
 *     -           -           - 5 (RW) relié à GND
 * D2  - pin  3(X) -
 *     - pin 13(X0)- broche 9  - lcd0 / 6 (EN) 
 *     - pin 14(X1)- broche 10 - lcd1 / 6 (EN) 
 *     - pin 15(X2)- broche 11 - lcd2 / 6 (EN) 
 *     - pin 12(X3)- broche 12 - lcd3 / 6 (EN) 
 *     - pin  1(X4)- broche 13 - lcd4 / 6 (EN) 
 *     - pin  5(X5)- broche 14 - lcd5 / 6 (EN) 
 *     - pin  2(X6)- broche 15 - lcd6 / 6 (EN) 
 *     - pin  4(X7)- broche 16 - lcd7 / 6 (EN) 
 * D6  -           - broche 2  - 11 (D4) 
 * D7  -           - broche 4  - 12 (D5)
 * D8  -           - broche 6  - 13 (D6)
 * D9  -           - broche 8  - 14 (D7)
 * D3  - pin 11(A) - 
 * D13 - pin 10(B) - (si plus de 2 lcd)
 * D10 - pin  9(C) - (si plus de 4 lcd)
 * D12 - pin  6(D) et transistor pour le 2° 4051 pin 6 (pour lcd 8 à15) (si plus de 8 lcd)
 * A0  -           - broche 7  - retour de BOUTON
 * A1  - potentiomètre du shield
 * A2  - diviseur de tension alimentation batterie
 * D11 - bouton poussoir du shield 
 * ___ - __________- _________ - ____________
 
 Les pin 1,2,4,5,12,13,14,15 doivent être tirées à la masse par une R de 10 à 22 kOhms
 En configuration 2, 4 ou 8 Lcd, tirer aussi pin 6 à la masse (par R ou direct)
 En configuration 2 ou 4 Lcd, tirer aussi pin 9 à la masse (par R ou direct)
 En configuration 2 Lcd, tirer aussi pin 10 à la masse (par R ou direct)
 ( 2 réseaux SIL de résistances de R103 peuvent faire l'affaire élégamment... )
*/
/* futur use !
#define clavier A0
#define potar A1
#define batterie A2
#define bouton 11
*/
//  LiquidCrystal lcd(RS, Enable,  D4, D5, D6, D7)
LiquidCrystal lcd(4, 2, 6, 7, 8, 9);

#define nb_afficheurs 4 // 2,4,8 ou 16
// tirer les sorties du 4051 à la masse avec une résistance de 10 à 22 kOhms
// pour 2 afficheurs mettre sur le 4051 B(10) et C(9) à la masse
// pour 4 afficheurs mettre sur le 4051 C(9) à la masse
// pour 16 afficheurs utiliser le INHIBIT(6)

// validations des 2 afficheurs par multiplexeur
#if nb_afficheurs == 2
int ger_enable[]= { 3 };
#endif

// validations des 4 afficheurs par multiplexeur
#if nb_afficheurs == 4
int ger_enable[]= { 3, 13 };
#endif

// validations des 8 afficheurs par multiplexeur
#if nb_afficheurs == 8
int ger_enable[]= { 3, 13, 10 };
#endif

// validations des 1- afficheurs par multiplexeur
#if nb_afficheurs == 16
int ger_enable[]= { 3, 13, 10, 12 };
#endif

// entrées-sorties
int led_a = 5;
// temporisations
int tempo_a,tempo_a_reset=500;
// GRAFCETS
int grafcet_a;
// calcul de vitesse ...
unsigned long tours_ps,tours;
int i;
float x;
char valueg[15];

#include "..\Goodfields_ger\ger_time.h"

#include "..\Goodfields_ger\ger_format.h"

#define creercaracterestype 1 //
//#define creercaracteresdemo 1 // si on veut un test au démarrage
#include "..\Goodfields_ger\ger_lcd_bargraph.h"

//**********************************************************
// positionne le curseur
// colonne = 0 à 15
// ligne = 0 à 7

// ligne 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15
// en[0] 0 0 1 1 0 0 1 1 0 0  1  1  0  0  1  1
// en[1] 0 0 0 0 1 1 1 1 0 0  0  0  1  1  1  1
// en[2] 0 0 0 0 0 0 0 0 1 1  1  1  1  1  1  1
// LIG%2 0 1 0 1 0 1 0 1 0 1  0  1  0  1  0  1

  void ger_lcd_setcursor (int colonne, int ligne) {
//    Serial && ligne && ((ligne & 0B00000100)?HIGH:LOW) && ((ligne & 0B00000010)?HIGH:LOW) && endli;
#if nb_afficheurs > 8
    digitalWrite(ger_enable[3], ligne & 0B00010000 );     
#endif
#if nb_afficheurs > 4
    digitalWrite(ger_enable[2], ligne & 0B00001000 );     
#endif
#if nb_afficheurs > 2
    digitalWrite(ger_enable[1], ligne & 0B00000100 );     
#endif
//    digitalWrite(ger_enable[0], ((ligne & 0B00000010)?HIGH:LOW) );    // 4 octets de plus 
    digitalWrite(ger_enable[0], ligne & 0B00000010 );     
//delay(1);  // inutile en fait
//    lcd.setCursor(colonne, ligne%2); // 8 octets de plus !
    lcd.setCursor(colonne, ligne & 0B00000001);
    return;
  }
//**********************************************************

//**********************************************************
// INITIALISATIONS
void setup() {
// mise à l'heure
  setTime(12,15,00,25,12,13); // 12h15 le 25 déc 2013 setupArduinoTime(2013,25,21,12,15,00));

// initialize the digital pins
  pinMode(led_a, OUTPUT);     

// définition taille du LCD  et inits :
//    pinMode(ger_valid, OUTPUT);     
    pinMode(ger_enable[0], OUTPUT);     
#if nb_afficheurs > 2
    pinMode(ger_enable[1], OUTPUT);     
#endif
#if nb_afficheurs > 4
    pinMode(ger_enable[2], OUTPUT);      
#endif
#if nb_afficheurs > 8
    pinMode(ger_enable[3], OUTPUT);     
#endif
   
// ligne série (pour traces)
  Serial.begin(9600);  // ou 9600 selon votre port COM
  Serial && versionx && " Serial ok" && endli;
//  Serial && "HIGH=" && HIGH && endli;
//  Serial && "LOW=" && LOW && endli;

// init LCD et affichage test et version
  for (i=0;i<nb_afficheurs;i++) {
    Serial && "init aff=" && i && " lignes " && i*2 && " et " && i*2+1 && endli; 
    ger_lcd_setcursor (0, i*2);
    lcd.begin(16, 2);
    ger_creercaracteres ();
    ger_lcd_setcursor (8, i*2); // 1ere ligne
    lcd && " Af" && i && "/" && nb_afficheurs && "  "; 
    lcd.setCursor(0, 1);  // 2ème ligne ou : ger_lcd_setcursor (0, i*2+1)
    lcd && versionx;     
  }

  delay(2500);

/*
// AFFICHAGE sur LCD de la version
  for (i=0;i<nb_afficheurs;i++) {
    ger_lcd_setcursor (0, i*2); lcd && versionx;     // 1ere ligne
    lcd.setCursor(0, 1); lcd && "GoodFields";    // 2eme ligne
    delay(500);        // pour profiter du LCD au démarrage
    lcd.clear();    // Effacement général LCD
  }
*/

// on arme les grafcets
  grafcet_a = 1;    // test basique
//  grafcet_a = 100;    // test led associée ou rétro-éclairage

}

//**********************************************************
// PROGRAMME PRINCIPAL
void loop() {

// tours par seconde
  tours++;

// pour allumer la led de l'afficheur utile
//    ger_lcd_setcursor (0, second()%8); //lcd && second()%8;
//  digitalWrite(lcd_enable,HIGH);
    
// gestion du temps
  ger_top_centieme ();
  
  if (top_10 > 0) {
    GER_TEMPO (tempo_a, top_10)
    top_10 = 0;
  }
// opérations à chaque dixième de seconde
  if (top_100ms) {
//    lcd.setCursor(0, 1);  lcd && " a=" && grafcet_a ;    // 2eme ligne lcd
    top_100ms = false;
  }
  
// opérations à chaque seconde
  if (top_1000ms) {
    top_1000ms = false;

    tours_ps=tours; tours=0;
    if (tours_ps < 2000) Serial && "ALERTE tours_ps=" && tours_ps && endli;
/*
ger_lcd_setcursor (0, 0); lcd && tours_ps && "t/s  ";

float x;
    for (i=1;i<6;i++) {
      ger_lcd_setcursor (0, i);
      lcd && ger_bargraph(second(),0.,59.,0.,59.,0.,59.,12,-3,'s',x);
    }

// horloge
    Serial && horodatage('h') && " top_seconde" && endli;

    ger_lcd_setcursor (0, 6); lcd && horodatage('d');
    ger_lcd_setcursor (0, 7); lcd && horodatage('h');
*/
    }

//--------------------------------
// GRAFCET A
switch (grafcet_a) {

// afficheur 0
  case 1:
  if ( tempo_a == 0 ) {        // si la tempo est terminée,
    Serial && "0" && endli;
    ger_lcd_setcursor (0, 0); lcd.clear();
    lcd && ger_bargraph(hour(),0.,24.,0.,24.,0.,24.,12,-2,'h',x) && " ";
//    ger_lcd_setcursor (0, 1); 
    lcd.setCursor (0, 1); lcd && ger_bargraph(minute(),0.,59.,0.,59.,0.,59.,12,-2,'m',x) && " ";
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a++;               // et on passe à l'étape suivante
  }
  break;      // cette instruction vous fait sauter à la fin du "switch"
  // sinon, on continue, et cela fait perdre du temps, et il peut arriver que d'autre "case" fonctionnent ! 

// afficheur 1
  case 2:
  if ( tempo_a == 0 ) {
    Serial && "1" && endli;
    ger_lcd_setcursor (0, 2); lcd && ger_bargraph(second(),0.,59.,0.,59.,0.,59.,12,-2,'s',x) && " ";
//    ger_lcd_setcursor (0, 3); lcd && horodatage('d');
// ou 
    lcd.setCursor (0, 1); lcd && horodatage('d');
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a++;
    if (nb_afficheurs < 4) grafcet_a=999;
  }    
  break;

// afficheur 2
  case 3:
  if ( tempo_a == 0 ) {
    Serial && "2" && endli;
    ger_lcd_setcursor (0, 4); 
                lcd && ger_bargraph( hour()*60+minute(), 0.,1440., 0.,120., 0.,120.,12,-2,'h',x);
    ger_lcd_setcursor (0, 5); lcd && tours && " x=" && x && "    ";
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a++;
  }    
  break;

// afficheur 3
  case 4:
  if ( tempo_a == 0 ) {
    Serial && "3" && endli;
    ger_lcd_setcursor (0, 6); lcd && ger_bargraph(second(),0.,59.,0.,59.,0.,59.,12,-2,'s',x) && " ";
    ger_lcd_setcursor (0, 7); lcd && ger_bargraph(milliseconde,0.,1000.,0.,1000.,0.,1000.,12,-3,'x',x);
    tempo_a = tempo_a_reset*10;             // on arme la tempo
    grafcet_a++;
    if (nb_afficheurs < 8) grafcet_a=999;
  }    
  break;

// afficheur 4 à 7, lignes 8 à 15
  case 5:
  if ( tempo_a == 0 ) {
    Serial && "4...7" && endli;
    for (i=8;i<16;i++) {
      ger_lcd_setcursor (0, i); 
      if (!(i & 0B0000001)) lcd.clear();
      lcd && "Ligne " && i && " " && second();
    }
    tempo_a = tempo_a_reset*10;             // on arme la tempo
    grafcet_a++;
    if (nb_afficheurs < 16) grafcet_a=999;
  }    
  break;

// afficheur 8 à 15, lignes 16 à 31
  case 6:
  if ( tempo_a == 0 ) {
    Serial && "8...15" && endli;
    for (i=16;i<32;i++) {
      ger_lcd_setcursor (0, i); 
      if (!(i & 0B0000001)) lcd.clear();
      lcd && "Ligne " && i && " " && second();
    }
    tempo_a = tempo_a_reset*10;             // on arme la tempo
    grafcet_a++;
  }    
  break;

// si grafcet_a n'a pas une valeur correcte, on passe ici
  default: 
      grafcet_a = 1;
  break;
  
  
//* * * * * * * * * * * * * * * * * * * * * * * * *  
// test afficheur et led associée à afficheur, sur le signal ENABLE
  case 100:
  case 101:
  case 102:
  case 103:

// pour allumer la led de l'afficheur en cours, on positionne le cusrseur en 0,i de façon répétitive
  i = grafcet_a-100;		// afficheur concerné
//    Serial && "voyant " && i && endli;
switch (i) {
  case 0:
    ger_lcd_setcursor (0, 0); // fixe
    break;
  case 1:
    if (milliseconde & 512)  ger_lcd_setcursor (0, 2); // clignotant lent 1/sec
    break;
  case 2:
    if (milliseconde & 256)  ger_lcd_setcursor (0, 4); // clignotant moyen 2/sec
    break;
  case 3:
    if (milliseconde & 64)  ger_lcd_setcursor (0, 6); // clignotant tres rapide 8/sec 
    break;
// pas de voyant
//  default:
//    break;
}

  if ( tempo_a == 0 ) {
    grafcet_a++;
    if (grafcet_a > 103) grafcet_a=100;
    Serial && "Afficheur " && grafcet_a-100 && endli;
    for (i=0;i<4;i++) {
      ger_lcd_setcursor (0, i*2); lcd.clear();
    }
    ger_lcd_setcursor (0, (grafcet_a-100)*2); lcd && ger_bargraph(second(),0.,59.,0.,59.,0.,59.,12,-2,'s',x) && " ";
//    ger_lcd_setcursor (0, 3); lcd && horodatage('d');
// ou 
    lcd.setCursor (0, 1); lcd && horodatage('d');
    tempo_a = 1500;             // on arme la tempo
  }    
  break;

} // fin du "switch" GRAFCET A
//*************************************************************
} // fin programme
//*************************************************************

